/* globals beaker */

/*
The webview has a set of sync method calls to the main process
which will stall the renderer if called while the main is
handling a workload.

https://github.com/electron/electron/blob/master/lib/renderer/web-view/web-view.js#L319-L371

This adds a set of async alternatives
*/

const methods = [
  'getURL',
  'loadURL',
  'getTitle',
  'isLoading',
  'isLoadingMainFrame',
  'isWaitingForResponse',
  'stop',
  'reload',
  'reloadIgnoringCache',
  'canGoBack',
  'canGoForward',
  'canGoToOffset',
  'clearHistory',
  'goBack',
  'goForward',
  'goToIndex',
  'goToOffset',
  'isCrashed',
  'setUserAgent',
  'getUserAgent',
  'openDevTools',
  'closeDevTools',
  'isDevToolsOpened',
  'isDevToolsFocused',
  'inspectElement',
  'setAudioMuted',
  'isAudioMuted',
  'undo',
  'redo',
  'cut',
  'copy',
  'paste',
  'pasteAndMatchStyle',
  'delete',
  'selectAll',
  'unselect',
  'replace',
  'replaceMisspelling',
  'findInPage',
  'stopFindInPage',
  'getId',
  'downloadURL',
  'inspectServiceWorker',
  'print',
  'printToPDF',
  'showDefinitionForSelection',
  'capturePage',
  'setZoomFactor',
  'setZoomLevel',
  'getZoomLevel',
  'getZoomFactor'
]

export default function addAsyncAlternatives (page) {
  methods.forEach(method => {
    page[method + 'Async'] = async (...args) => {
      if (!page.isWebviewReady) return false
      return beaker.browser.doWebcontentsCmd(method, page.wcID, ...args)
    }
  })
}
